@@ -12,6 +12,7 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <time.h>
 #include <string.h>
 #include <assert.h>
 #include <sys/types.h>
@@ -20,15 +21,19 @@
 
 #ifndef __PARSING_H__
 #define __PARSING_H__
-#define __PARSING_VERSION__ "0.3.6"
+#define __PARSING_VERSION__ "0.4.0"
 
 /**
- * == libparsing
- * == C & Python Parsing Element Grammar Library
- * -- Version: 0.3.0
- * -- URL: http://github.com/sebastien/parsing
+ * #  libparsing
+ * ## Python Parsing Element Grammar Library
  *
- * #START:INTRO
+ * ```
+ * Version :  0.3.6
+ * URL     :  http://github.com/sebastien/parsing
+ * README  :  https://cdn.rawgit.com/sebastien/libparsing/master/README.html
+ * ```
+ *
+ * [START:INTRO]
  *
  * `libparsing` is a parsing element grammar (PEG) library written in C with
  * Python bindings. It offers decent performance while allowing for a
@@ -98,25 +103,29 @@
  * Match* match = Grammar_parseString(g, "10 + 20 / 5")
  * ```
  *
- * #END:INTRO
+ * [END:INTRO]
  *
  * Installing
  * ==========
  *
  * To install the Python parsing module:
  *
- * >	easy_install libparsing    # From Setuptools
- * >	pip install  libparsing     # From PIP
+ * ```shell
+ * easy_install libparsing    # From Setuptools
+ * pip install  libparsing     # From PIP
+ * ```
  *
  * Note that for the above to work, you'll need a C compiler and libpcre-dev.
  * On Ubuntu, do `sudo apt install build-essential libprcre-dev`.
  *
  * To compile the C parsing module:
  *
- * >	git clone http://github.com/sebastien/libparsing
- * >	cd libparsing
- * >	make
- * >	make install               # You can set PREFIX
+ * ```shell
+ * git clone http://github.com/sebastien/libparsing
+ * cd libparsing
+ * make
+ * make install               # You can set PREFIX
+ * ```
  *
  * `libparsing` works with GCC4 and Clang and is written following the `c11`
  * standard.
@@ -151,7 +160,9 @@ typedef ITERATION_UNIT iterated_t;
  *
  * You can get an iterator on a file by doing:
  *
- * >	Iterator* iterator = Iterator_Open("example.txt");
+ * ```c
+ * Iterator* iterator = Iterator_Open("example.txt");
+ * ```
  *
 */
 
@@ -255,6 +266,7 @@ bool FileInput_move   ( Iterator* this, int n );
 
 typedef struct ParsingContext ParsingContext;
 typedef struct ParsingElement ParsingElement;
+typedef struct ParsingResult  ParsingResult;
 typedef struct Reference      Reference;
 typedef struct Match          Match;
 
@@ -264,6 +276,7 @@ typedef struct Grammar {
 	ParsingElement*  skip;        // The skipped element
 	int              axiomCount;  // The count of parsing elemetns in axiom
 	int              skipCount;   // The count of parsing elements in skip
+	bool             isVerbose;
 } Grammar;
 
 
@@ -277,10 +290,10 @@ void Grammar_free(Grammar* this);
 void Grammar_prepare ( Grammar* this );
 
 // @method
-Match* Grammar_parseFromIterator( Grammar* this, Iterator* iterator );
+ParsingResult* Grammar_parseFromIterator( Grammar* this, Iterator* iterator );
 
 // @method
-Match* Grammar_parseFromPath( Grammar* this, const char* path );
+ParsingResult* Grammar_parseFromPath( Grammar* this, const char* path );
 
 /**
  * Elements
@@ -300,8 +313,7 @@ int Element_walk( Element* this, WalkingCallback callback );
 int Element__walk( Element* this, WalkingCallback callback, int step );
 
 /**
- * 1. Parsing Elements
- * -------------------
+ * ### Parsing Elements
  *
  * Parsing elements are the core elements that recognize and process input
  * data. There are 4 basic types: `Work`, `Token`, `Group` and `Rule`.
@@ -339,6 +351,10 @@ typedef struct Match {
 // @define
 #define STATUS_MATCHED     'M'
 // @define
+#define STATUS_SUCCESS     'S'
+// @define
+#define STATUS_PARTIAL     's'
+// @define
 #define STATUS_FAILED      'X'
 // @define
 #define STATUS_INPUT_ENDED '.'
@@ -446,8 +462,7 @@ Match* ParsingElement_process( ParsingElement* this, Match* match );
 ParsingElement* ParsingElement_name( ParsingElement* this, const char* name );
 
 /**
- * 2. Word
- * -------
+ * ### Word
  *
  * Words recognize a static string at the current iterator location.
  *
@@ -469,8 +484,7 @@ ParsingElement* Word_new(const char* word);
 Match*          Word_recognize(ParsingElement* this, ParsingContext* context);
 
 /**
- * 3. Tokens
- * ---------
+ * ### Tokens
  *
  * Tokens are regular expression based parsing elements. They do not have
  * any children and test if the regular expression matches exactly at the
@@ -513,8 +527,7 @@ void TokenMatch_free(Match* match);
 const char* TokenMatch_group(Match* match, int index);
 
 /**
- * 4. References
- * -------------
+ * ### References
  *
  * We've seen that parsing elements can have `children`. However, a parsing
  * element's children are not directly parsing elements but rather
@@ -582,8 +595,7 @@ int Reference__walk( Reference* this, WalkingCallback callback, int step );
 Match* Reference_recognize(Reference* this, ParsingContext* context);
 
 /**
- * 5. Groups
- * ---------
+ * ### Groups
  *
  * Groups are composite parsing elements that will return the first matching reference's
  * match. Think of it as a logical `or`.
@@ -596,8 +608,7 @@ ParsingElement* Group_new(Reference* children[]);
 Match*          Group_recognize(ParsingElement* this, ParsingContext* context);
 
 /**
- * 6. Rules
- * --------
+ * ### Rules
  *
  * Groups are composite parsing elements that only succeed if all their
  * matching reference's.
@@ -610,8 +621,7 @@ ParsingElement* Rule_new(Reference* children[]);
 Match*          Rule_recognize(ParsingElement* this, ParsingContext* context);
 
 /**
- * 7. Procedures
- * -------------
+ * ### Procedures
  *
  * Procedures are parsing elements that do not consume any input, always
  * succeed and usually have a side effect, such as setting a variable
@@ -631,8 +641,7 @@ ParsingElement* Procedure_new(ProcedureCallback c);
 Match*          Procedure_recognize(ParsingElement* this, ParsingContext* context);
 
 /*
- * 8. Conditions
- * -------------
+ * ### Conditions
  *
  * Conditions, like procedures, execute arbitrary code when executed, but
  * they might return a FAILURE.
@@ -662,14 +671,51 @@ Match*          Condition_recognize(ParsingElement* this, ParsingContext* contex
 typedef struct ParsingStep    ParsingStep;
 typedef struct ParsingOffset  ParsingOffset;
 
-// @type ParsingContext
+// @type
+typedef struct ParsingStats {
+	size_t  bytesRead;
+	double  parseTime;
+	size_t* successBySymbol;
+	size_t* failureBySymbol;
+} ParsingStats;
+
+// @constructor
+ParsingStats* ParsingStats_new(void);
+
+// @destructor
+void ParsingStats_free(ParsingStats* this);
+
+// @method
+void ParsingStats_setSymbolsCount(ParsingStats* this, size_t t);
+
+// @type
 typedef struct ParsingContext {
 	struct Grammar*              grammar;      // The grammar used to parse
 	struct Iterator*             iterator;     // Iterator on the input data
 	struct ParsingOffset* offsets;      // The parsing offsets, starting at 0
 	struct ParsingOffset* current;      // The current parsing offset
+	struct ParsingStats*  stats;
 } ParsingContext;
 
+// @constructor
+ParsingContext* ParsingContext_new( Grammar* g, Iterator* iterator );
+
+// @destructor
+void ParsingContext_free( ParsingContext* this );
+
+// @type
+typedef struct ParsingResult {
+	char            status;
+	Match*          match;
+	ParsingContext* context;
+} ParsingResult;
+
+// @constructor
+ParsingResult* ParsingResult_new(Match* match, ParsingContext* context);
+
+// @method
+void ParsingResult_free(ParsingResult* this);
+
 /*
  * The result of _recognizing_ parsing elements at given offsets within the
  * input stream is stored in `ParsingOffset`. Each parsing offset is a stack
